% -----------------------------------------------------------------------------------
% Copyright (c) <2016>, <Information-technology Promotion Agency, Japan>
% All rights reserved.
% 
% Redistribution and use in source and binary forms, with or without modification,
% are permitted provided that the following conditions are met:
% 
% 1. Redistributions of source code must retain the above copyright notice,
%    this list of conditions and the following disclaimer.
% 
% 2. Redistributions in binary form must reproduce the above copyright notice,
%    this list of conditions and the following disclaimer in the documentation
%    and/or other materials provided with the distribution.

% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
% ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
% WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
% IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
% INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
% BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
% DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
% LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
% OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
% ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
% -----------------------------------------------------------------------------------
%% draw_Circle ~`
% e~̕`s܂B
%%
classdef draw_Circle
    %%
    %%% vpeB
    properties
        th = (0:.02:1)*2*pi;
        x = 0;
        y = 0;
        r = 1;
        % F
        color = common.colorW;
        edgeColor = common.colorK;
        % nh[
        hndr = 0;
        % `^Cv
        dtype = 0;        
    end
    
    %% \bh
    methods
        %% obj = draw_Circle(x, y, r, cc) RXgN^
        %  
        %    x      FxW
        %    y      FyW
        %    r      Fa
        %    cc     FF
        function obj = draw_Circle(x, y, r, cc)
            obj.x = x;
            obj.y = y;
            obj.r = r;
            obj.edgeColor = cc;
            obj.color = cc;
        end
        
        %%  hndr = fillCircle() Fh~`
        %  
        %
        %  Ԃl
        %    hndr   F`ẽnh[
        function hndr = fillCircle(obj)
            obj.dtype = 'fill';
            try
                delete(obj.hndr);
            end
            xParam = obj.r*cos(obj.th)+obj.x;
            yParam = obj.r*sin(obj.th)+obj.y;
            obj.hndr = fill(xParam, yParam, obj.color,...
                'EdgeColor', obj.edgeColor);
            hndr = obj.hndr;
        end
        
        %%  hndr = drawCircle() ~`
        %  
        %
        %  Ԃl
        %    hndr   F`ẽnh[
        function hndr = drawCircle(obj)
            obj.dtype = 'draw';
            try
                delete(obj.hndr);
            end
            xParam = obj.r*cos(obj.th)+obj.x;
            yParam = obj.r*sin(obj.th)+obj.y;
            obj.hndr = plot(xParam, yParam, 'Color', obj.edgeColor);
            hndr = obj.hndr;
        end
        
        %%  hndr = drawCrossCircle() X~`
        %  
        %
        %  Ԃl
        %    hndr   F`ẽnh[
        function hndr = drawCrossCircle(obj)
            obj.dtype = 'drawcross';
            try
                delete(obj.hndr);
            end
            ccHndr1 = obj.drawCircle;
            xParam = obj.r*cos(pi/4)+obj.x;
            yParam = obj.r*sin(pi/4)+obj.y;
            ccHndr2 = plot([xParam, -xParam + 2*obj.x], [yParam, -yParam+2*obj.y], 'Color', obj.edgeColor);
            ccHndr3 = plot([-xParam + 2*obj.x, xParam], [yParam, -yParam+2*obj.y], 'Color', obj.edgeColor);
            obj.hndr = [ccHndr1 ccHndr2 ccHndr3];
            hndr = obj.hndr;
        end
        
        %%  hndr = drawHalfCircle(varargin) ~`
        %  
        %   varargin   :ϕϐ
        %       (1) 
        %               0:
        %               1:
        %               2:
        %               3:E
        %  Ԃl
        %    hndr   F`ẽnh[
        function hndr = drawHalfCircle(obj, varargin)
            thBuf = obj.th;
            obj.th=[0:.04:1,0]*pi;
            obj.hndr = obj.drawCircle;
            if nargin >= 2
                rect = cellmat(varargin(1));
            else
                rect = 0;
            end
            rotate(obj.hndr,[0,0,1],90*mod(rect,4),[obj.x,obj.y,0]);
            obj.th = thBuf;
            obj.dtype = 'drawhalf';
            hndr = obj.hndr;
        end
        
        %%  hndr = fillHalfCircle(varargin) ~`
        %  
        %   varargin   :ϕϐ
        %       (1) 
        %               0:
        %               1:
        %               2:
        %               3:E
        %  Ԃl
        %    hndr   F`ẽnh[
        function hndr = fillHalfCircle(obj, varargin)
            thBuf = obj.th;
            obj.th=[0:.04:1,0]*pi;
            obj.hndr = obj.fillCircle;
            if nargin >= 2
                rect = cell2mat(varargin(1));
            else
                rect = 0;
            end
            rotate(obj.hndr,[0,0,1],90*mod(rect,4),[obj.x,obj.y,0]);
            obj.th = thBuf;
            obj.dtype = 'fillhalf';
            hndr = obj.hndr;
        end
        
        %%  setColor(color) Fݒ
        %  
        %   color   :ݒF
        function obj = setColor(obj, color)
            obj.color = color;
            obj.redraw;
        end
        
        %%  setEdgeColor(edgeColor) GbWFݒ
        %  
        %   edgeColor   :ݒF
        function obj = setEdgeColor(obj, edgeColor)
            obj.edgeColor = edgeColor;
            obj.redraw;
        end
    end
    
    %% veNg\bh
    methods(Access = protected)
        %%  redraw() dtypȅɊÂčĕ`悷B     
        function obj = redraw(obj)
            if obj.hndr ~= 0
                switch obj.dtype
                    case 'fill'
                        obj = obj.fillCircle;
                    case 'draw'
                        obj = obj.draw_Circle;
                    case 'drawcross'
                        obj = obj.drawCrossCircle;
                    case 'drawhalf'
                        obj = obj.drawHalfCircle;
                    case 'fillhalf'
                        obj = obj.fillHalfCircle;
                end
            end
        end
    end
end

